/*************************************************************************************
Data Visualization               
Tested in Processing 4                                   
                                                     
Author: SIMING, SU                       
Supervisor: George Legrady                               

Usage: 1. A mouse left-drag will rotate the camera around the subject.
          2. A right drag will zoom in and out. 
          3. A middle-drag (command-left-drag on mac) will pan. 
          4. A float-click restores the camera to its original position. 
          5. The shift key constrains rotation and panning to one axis or the other.
*************************************************************************************/

import peasy.*;
import controlP5.*;
ControlP5 cp5;
PeasyCam cam;

/* 3D portion */
int startYear = 2015;
int endYear = 2021;
int loop = 0;

String mystring = "";
boolean all;
color mycolor;

Table cinout_table;
int numRows, numColumns;

ArrayList<Date> dates = new ArrayList<Date>();



/* 4D portion */
float theta = 0;
boolean dim_switch = true; 
float[][][] projected_3d = new float[16][1][3];
float theta_add = 0.01;
boolean change = false;

void setup(){
  size(1300, 1300, P3D);
  cam = new PeasyCam(this, 200);
  cp5 = new ControlP5(this);
  
  cp5.addToggle("dim_switch").setPosition(50,50); 
  cp5.setAutoDraw(false);
  
  
  //Added on Jan 30th by Rodger Luo
  float  cameraZ = (height/2.0) / tan(PI*60.0/360.0);
  perspective(PI/3.0, width/height, 0.001, cameraZ*10.0); //set near clip to 0.001 to go inside data
  //Finished adding 
 
  cinout_table = loadTable("cin_cout_info.csv", "header");
  numRows = cinout_table.getRowCount();
  println("Rows: " + numRows);
  for(int i=0; i<numRows; i++){
    int cin_year = cinout_table.getInt(i,0);
    int cin_month = cinout_table.getInt(i,1);
    int cin_day = cinout_table.getInt(i,2);
    int cout_year = cinout_table.getInt(i,3);
    int cout_month = cinout_table.getInt(i,4);
    int cout_day = cinout_table.getInt(i,5);
    String type = cinout_table.getString(i,6);
    dates.add(new Date(cin_day, cin_month, cin_year, 
    cout_day, cout_month, cout_year, type));
  }
}

void draw(){
  if (dim_switch){
    background(100);
    for (int j = 0; j < dates.size(); j++){
      float time = 1000;
      float cout_yearaxis = map(dates.get(j).cout_year, startYear, endYear, -50, 50);
      float cout_monthaxis = map(dates.get(j).cout_month, 1, 12, -50, 50);
      float cout_dayaxis = map(dates.get(j).cout_day, 1, 31, -50, 50);
      
      float cin_yearaxis = map(dates.get(j).cin_year, startYear, endYear, -50, 50);
      float cin_monthaxis = map(dates.get(j).cin_month, 1, 12, -50, 50);
      float cin_dayaxis = map(dates.get(j).cin_day, 1, 31, -50, 50);
      
      float day_rate = (cin_dayaxis - cout_dayaxis)/time;
      float month_rate = (cin_monthaxis - cout_monthaxis)/(time);
      float year_rate = (cin_yearaxis - cout_yearaxis)/(time);
      
      String callNumber = dummy(dates.get(j).type);
      
      cout_dayaxis += loop * day_rate;
      cout_monthaxis += loop * month_rate;
      cout_yearaxis += loop * year_rate;
      
      if ((day_rate <= 0) && (cout_dayaxis <= cin_dayaxis)){
        cout_dayaxis = cin_dayaxis;
      }
      if ((day_rate >= 0) && (cout_dayaxis >= cin_dayaxis)){
        cout_dayaxis = cin_dayaxis;
      }
      if ((month_rate >= 0) && (cout_monthaxis >= cin_monthaxis)){
        cout_monthaxis = cin_monthaxis;
      }
      if ((month_rate <= 0) && (cout_monthaxis <= cin_monthaxis)){
        cout_monthaxis = cin_monthaxis;
      }
      if (cout_yearaxis >= cin_yearaxis){
        cout_yearaxis = cin_yearaxis;
      }
      
      if (all){
        if (callNumber.equals("Adele")){
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, #003f5c);
        }
        else if (callNumber.equals("Brown")){
          draw_points(cout_dayaxis, cout_monthaxis , cout_yearaxis, #58508d);
        }
        else if (callNumber.equals("Kidz")){
          draw_points(cout_dayaxis, cout_monthaxis , cout_yearaxis,  #ff6361);
        }
        else if (callNumber.equals("Compilation")){
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, #bc5090);
        }
        else{
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, #ffa600);
        }
      }
      
      else{
        if (callNumber.equals(mystring)){
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, mycolor);
         }
      }
    }
    stroke(255);
    noFill();
    box(100);
    drawLabels();
    
   }
   
 /* 4d switch*/   
   else{
     float[][] rotation_matZW = {
      {cos(theta), -sin(theta), 0, 0}, 
      {sin(theta), cos(theta), 0, 0},
      {0, 0, 1, 0}, 
      {0, 0, 0, 1}};
      
    float[][] rotation_matXY = {
      {1, 0, 0, 0},
      {0, 1, 0, 0},
      {0, 0, cos(theta), -sin(theta)}, 
      {0, 0, sin(theta), cos(theta)}
    };
    
    float[][][] rotation_matrices = {rotation_matZW, rotation_matXY};
    
     background(100);
      for (int j = 0; j < dates.size(); j++){
        float time_4d = 1000;
        float cout_yearaxis = map(dates.get(j).cout_year, startYear, endYear, -1, 1);
        float cout_monthaxis = map(dates.get(j).cout_month, 1, 12, -1, 1);
        float cout_dayaxis = map(dates.get(j).cout_day, 1, 31, -1, 1);
        float cout_numaxis = map(dummy_num(dates.get(j).type), 1, 5, -1, 1);
        int cout_checkaxis = dummy_num(dates.get(j).type);
       
        float cin_yearaxis = map(dates.get(j).cin_year, startYear, endYear, -1, 1);
        float cin_monthaxis = map(dates.get(j).cin_month, 1, 12, -1, 1);
        float cin_dayaxis = map(dates.get(j).cin_day, 1, 31, -1, 1);
        
        
        float[][] vec_cout = {{cout_dayaxis, cout_monthaxis, cout_yearaxis, cout_numaxis}};
        int [] indices = {0,1};
        float[][] rotated_cout = rotate_it(vec_cout, rotation_matrices, indices);
        float[][] projected_cout = project_it(rotated_cout);
        
        float[][] vec_cin= {{cin_dayaxis, cin_monthaxis, cin_yearaxis, cout_numaxis}};
        float[][] rotated_cin = rotate_it(vec_cin, rotation_matrices, indices);
        float[][] projected_cin = project_it(rotated_cin);
        
        cout_dayaxis = projected_cout[0][0];
        cout_monthaxis = projected_cout[0][1]; 
        cout_yearaxis = projected_cout[0][2];
        cin_dayaxis = projected_cin[0][0];
        cin_monthaxis = projected_cin[0][1];
        cin_yearaxis = projected_cin[0][2];
        
        float day_rate = (cin_dayaxis - cout_dayaxis)/time_4d;
        float month_rate = (cin_monthaxis - cout_monthaxis)/(time_4d);
        float year_rate = (cin_yearaxis - cout_yearaxis)/(time_4d);

        cout_dayaxis += loop * day_rate;
        cout_monthaxis += loop * month_rate;
        cout_yearaxis += loop * year_rate;
        
        if ((day_rate <= 0) && (cout_dayaxis <= cin_dayaxis)){
          cout_dayaxis = cin_dayaxis;
        }
        if ((day_rate >= 0) && (cout_dayaxis >= cin_dayaxis)){
          cout_dayaxis = cin_dayaxis;
        }
        if ((month_rate >= 0) && (cout_monthaxis >= cin_monthaxis)){
          cout_monthaxis = cin_monthaxis;
        }
        if ((month_rate <= 0) && (cout_monthaxis <= cin_monthaxis)){
          cout_monthaxis = cin_monthaxis;
        }
        if ((year_rate >= 0) && (cout_yearaxis >= cin_yearaxis)){ 
          cout_yearaxis = cin_yearaxis;
        }
        if ((year_rate <= 0) && (cout_yearaxis <= cin_yearaxis)){ 
          cout_yearaxis = cin_yearaxis;
        }
        
        cout_dayaxis *= 40;
        cout_monthaxis *= 40;
        cout_yearaxis *= 40;

        if (cout_checkaxis == 1){
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, #003f5c);
        }
        else if (cout_checkaxis  == 2){
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, #58508d);
        }
        else if (cout_checkaxis  == 3){
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, #ff6361);
        }
        else if (cout_checkaxis  == 4){
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, #bc5090);
        }
        else{
          draw_points(cout_dayaxis, cout_monthaxis, cout_yearaxis, #ffa600);
        }
        
        //float day_rate = (cin_dayaxis - cout_dayaxis)/time;
        //float month_rate = (cin_monthaxis - cout_monthaxis)/(time);
        //float year_rate = (cin_yearaxis - cout_yearaxis)/(time);
      }
     draw_tesseract(rotation_matrices);
   }
 
 
 
 // fill(20, 20);
 // noStroke();
 // box(100);
  
  loop += 1; 
  theta += theta_add;
  gui();
}





void keyPressed(){
    if(key == '1'){
      loop = 0;
      all = false;
      mystring = "Adele";
      mycolor = #003f5c;
    }
    else if(key == '2'){
      loop = 0;
      all = false;
      mystring = "Brown";
      mycolor = #58508d;
    }
    else if(key == '3'){
      loop = 0;
      all = false;
      mystring = "Kidz";
      mycolor = #ff6361;
    }
    else if(key == '4'){
      loop = 0;
      all = false;
      mystring = "Compilation";
      mycolor = #bc5090;
    }
    else if(key == '5'){
      loop = 0;
      all = false;
      mystring = "Trevor";
      mycolor = #ffa600;
    }
    
    if(key == 'a'){
      loop = 0;
      all = true;
    }
    
    if(key == ' '){
      change = !change;
      if (change){
        theta_add = 0;
      }
      else{
        theta_add = 0.01;
      }
    }
}
